;=========================================================================
; This program creates and displays the dot-matrix digit bitmaps
; that the dot-matrix digital clock demo uses.
;
; The dot bitmap that the digits are built from was created in
; Paint, and the 24-bpp color depth was chosen so the dot could
; be reasonably anti-aliased.
;=========================================================================

include \GeneSys\include\GeneSysRt.inc
include \GeneSys\macros\macros.asm
include xmacros.asm
include imdialog.asm
include support.asm
include patterns.asm

IDC_0     equ 100
IDC_1     equ 101
IDC_2     equ 102
IDC_3     equ 103
IDC_4     equ 104
IDC_5     equ 105
IDC_6     equ 106
IDC_7     equ 107
IDC_8     equ 108
IDC_9     equ 109

;=========================================================================
.data
    dtuX        dd      0
    dtuY        dd      0
    hbmpDot     HBITMAP 0
    hbmpDigits  HBITMAP 10 dup(0)
.code
;=========================================================================

;----------------------------------------------------------------
; This procedure creates a Device Dependent Bitmap (DDB) for the
; specified digit, copies the bitmap to a file (as a DIB), and
; loads the bitmap into the appropriate bitmap control. The digit
; dot patterns are defined in patterns.asm.
;----------------------------------------------------------------

CreateDigit proc uses ebx edi esi digit:DWORD,
                                  hwndBitmap:HWND,
                                  pszFilename:DWORD

    LOCAL hdcDot:HDC, hdcBm:HDC, hdcScreen:HDC
    LOCAL hbmp:HBITMAP
    LOCAL rc:RECT

    ;-----------------------------------------------------------------
    ; Get a DC for the screen, create a compatible memory DC for the
    ; dot bitmap and a compatible memory DC and compatible bitmap for
    ; the digit.
    ;-----------------------------------------------------------------

    invoke GetDC, NULL
    mov hdcScreen, eax

    invoke CreateCompatibleDC, hdcScreen
    mov hdcDot, eax
    invoke CreateCompatibleDC, hdcScreen
    mov hdcBm, eax
    invoke CreateCompatibleBitmap, hdcScreen, 84, 106
    mov hbmp, eax

    ;-----------------------------------------------------------
    ; In addition to storing the digit bitmap handle in a local
    ; variable, we need to store it in a global array so we can
    ; delete it when we finish using it. And because we need
    ; the handle to display the bitmap, we will not be finished
    ; using it until the we are ready to terminate.
    ;-----------------------------------------------------------

    mov edx, digit
    mov [hbmpDigits+edx], eax

    ;-----------------------------------------------------------
    ; We are finished with the DC for the screen so release it.
    ;-----------------------------------------------------------

    invoke ReleaseDC, NULL, hdcScreen

    ;-------------------------------------------------------
    ; Select our bitmaps into the corresponding memory DCs.
    ;-------------------------------------------------------

    invoke SelectObject, hdcBm, hbmp
    invoke SelectObject, hdcDot, hbmpDot

    ;------------------------------------------------------------
    ; Fill the digit bitmap with black. Note that the right and
    ; bottom coordinates have been extended by one pixel each so
    ; the bottom row and rightmost column will contain black
    ; instead of random garbage. See the Microsoft documentation
    ; for the FillRect function for more information.
    ;------------------------------------------------------------

    invoke SetRect, ADDR rc, 0, 0, 84, 106
    invoke GetStockObject, BLACK_BRUSH
    invoke FillRect, hdcBm, ADDR rc, eax

    ;-----------------------------------------------
    ; Set EBX to the pattern for the current digit.
    ;-----------------------------------------------

    mov ebx, OFFSET patterns
    mov eax, digit
    imul eax, 35
    add ebx, eax

    ;------------------------------------------------------------
    ; Step through the pattern and for each instance of '@' copy
    ; the dot to the correct location in the target bitmap.
    ;------------------------------------------------------------

    xor edi, edi
    .WHILE edi <= 6
      xor esi, esi
      .WHILE esi <= 4
        mov edx, edi
        imul edx, 5
        add edx, esi
        mov al, [ebx+edx]
        .IF al == '@'
          mov eax, edi
          imul eax, 14
          add eax, 4
          mov edx, esi
          imul edx, 14
          add edx, 7
          invoke BitBlt, hdcBm, edx, eax, 84, 106, hdcDot, 0, 0, SRCCOPY
        .ENDIF
        inc esi
      .ENDW
      inc edi
    .ENDW

    ;-----------------------------------------------------------------
    ; Create the bitmap file preserving the 24-bpp of the dot bitmap.
    ;-----------------------------------------------------------------

    invoke CreateBitmapFile, hbmp, 24, pszFilename

    ;----------------------------------------------------------------
    ; Load the completed digit into the specified SS_BITMAP control.
    ;----------------------------------------------------------------

    SETIMAGE hwndBitmap, hbmp

    ;----------------------
    ; Clean up and return.
    ;----------------------

    invoke DeleteDC, hdcDot
    invoke DeleteDC, hdcBm

    ret
CreateDigit endp

;=========================================================================

DeleteBitmapResources proc

    invoke DeleteObject, hbmpDot
    invoke DeleteObject, hbmpDigits+4*0
    invoke DeleteObject, hbmpDigits+4*1
    invoke DeleteObject, hbmpDigits+4*2
    invoke DeleteObject, hbmpDigits+4*3
    invoke DeleteObject, hbmpDigits+4*4
    invoke DeleteObject, hbmpDigits+4*5
    invoke DeleteObject, hbmpDigits+4*6
    invoke DeleteObject, hbmpDigits+4*7
    invoke DeleteObject, hbmpDigits+4*8
    invoke DeleteObject, hbmpDigits+4*9

    ret

DeleteBitmapResources endp

;=========================================================================

DialogProc proc hwndDlg:HWND, uMsg:UINT, wParam:WPARAM, lParam:LPARAM

    LOCAL hdcBM:HDC
    LOCAL ps:PAINTSTRUCT

    SWITCH uMsg

      CASE WM_INITDIALOG

        ;----------------------------------------------------------------
        ; Adjust the client area to the specified pixel size, precisely.
        ;----------------------------------------------------------------

        invoke SetClientSize, hwndDlg, (5*14+14)*10+8, (7*14+16)

        ;--------------------------------------------------------
        ; Load the dot bitmap from its file and save the handle.
        ;--------------------------------------------------------

        invoke LoadImage, NULL, chr$("dot.bmp"),
                          IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE
        mov hbmpDot, eax

        ;----------------------------------------------------
        ; For each of the SS_BITMAP static controls, get the
        ; handle and pass it, along with a digit number and
        ; a filename, to the CreateDigit procedure.
        ;----------------------------------------------------

        invoke GetDlgItem, hwndDlg, IDC_0
        invoke CreateDigit, 0, eax, chr$("d0.bmp")
        invoke GetDlgItem, hwndDlg, IDC_1
        invoke CreateDigit, 1, eax, chr$("d1.bmp")
        invoke GetDlgItem, hwndDlg, IDC_2
        invoke CreateDigit, 2, eax, chr$("d2.bmp")
        invoke GetDlgItem, hwndDlg, IDC_3
        invoke CreateDigit, 3, eax, chr$("d3.bmp")
        invoke GetDlgItem, hwndDlg, IDC_4
        invoke CreateDigit, 4, eax, chr$("d4.bmp")
        invoke GetDlgItem, hwndDlg, IDC_5
        invoke CreateDigit, 5, eax, chr$("d5.bmp")
        invoke GetDlgItem, hwndDlg, IDC_6
        invoke CreateDigit, 6, eax, chr$("d6.bmp")
        invoke GetDlgItem, hwndDlg, IDC_7
        invoke CreateDigit, 7, eax, chr$("d7.bmp")
        invoke GetDlgItem, hwndDlg, IDC_8
        invoke CreateDigit, 8, eax, chr$("d8.bmp")
        invoke GetDlgItem, hwndDlg, IDC_9
        invoke CreateDigit, 9, eax, chr$("d9.bmp")

      CASE WM_CTLCOLORDLG

        ;------------------------------------------
        ; Set the client area background to black.
        ;------------------------------------------

        invoke GetStockObject, BLACK_BRUSH
        ret

      CASE WM_COMMAND

        SWITCH loword(wParam)

          CASE IDCANCEL

            ;--------------------
            ; Clean up and exit.
            ;--------------------

            invoke DeleteBitmapResources
            invoke EndDialog, hwndDlg, NULL

        ENDSW

      CASE WM_CLOSE

        ;--------------------
        ; Clean up and exit.
        ;--------------------

        invoke DeleteBitmapResources
        invoke EndDialog, hwndDlg, NULL

    ENDSW

    xor eax, eax
    ret

DialogProc endp

;=========================================================================
start:
;=========================================================================

    ;---------------------------------------------------------------
    ; Get a width and height in DTUs that will size the client area
    ; close to the specified pixel width and height, regardless of
    ; the system font settings.
    ;---------------------------------------------------------------

    invoke ClientPixelWidthToDialogWidth, (5*14+14)*10+8
    mov dtuX, eax
    invoke ClientPixelHeightToDialogHeight, 7*14+16
    mov dtuY, eax

    invoke DefineDialog, chr$("Create Dot Matrix BMPs"),
                         WS_OVERLAPPED or WS_SYSMENU \
                         or DS_CENTER,
                         0, 0, dtuX, dtuY,
                         0, 0,
                         10

    ;----------------------------------------------------------
    ; Specify the SS_BITMAP static control positions in pixels
    ; converted to DTUs, so the position will not vary with the
    ; system font settings.
    ;----------------------------------------------------------

    invoke DefineBitmap, 0, pdx(6+84*0), pdy(4), 0, IDC_0
    invoke DefineBitmap, 0, pdx(6+84*1), pdy(4), 0, IDC_1
    invoke DefineBitmap, 0, pdx(6+84*2), pdy(4), 0, IDC_2
    invoke DefineBitmap, 0, pdx(6+84*3), pdy(4), 0, IDC_3
    invoke DefineBitmap, 0, pdx(6+84*4), pdy(4), 0, IDC_4
    invoke DefineBitmap, 0, pdx(6+84*5), pdy(4), 0, IDC_5
    invoke DefineBitmap, 0, pdx(6+84*6), pdy(4), 0, IDC_6
    invoke DefineBitmap, 0, pdx(6+84*7), pdy(4), 0, IDC_7
    invoke DefineBitmap, 0, pdx(6+84*8), pdy(4), 0, IDC_8
    invoke DefineBitmap, 0, pdx(6+84*9), pdy(4), 0, IDC_9

    invoke CreateModalDialog, 0, ADDR DialogProc, 0

    invoke ExitProcess, 0

;=========================================================================

end start
