;=========================================================================
;; This module contains procedures and definitions to support the
;; creation of a dialog box template in allocated memory, and the
;; creation of a modal or modeless dialog from the template.
;;
;; The dialog box template consists of a DLGTEMPLATE structure
;; followed by three or four variable-length arrays, followed by
;; zero or more DLGITEMTEMPLATE structures each followed by three
;; variable-length arrays. The DLGTEMPLATE structure and the
;; associated arrays define the dialog window. The DLGITEMTEMPLATE
;; structures and the associated arrays define the controls in the
;; dialog.
;;
;; The variable-length arrays consist of WORD (16-bit) elements.
;; The first three arrays following the DLGTEMPLATE structure
;; specify the menu, class, and title for the dialog. The three
;; arrays following each DLGITEMTEMPLATE structure specify the
;; class, title, and creation data for the control. Each of these
;; arrays will have at least one element, and the system will
;; interpret the contents of the array based on the value of the
;; first element. For the dialog menu, class, and title arrays,
;; and the control creation data array, if the first element is
;; zero, then the array is effectively empty and there are no
;; other elements. For the dialog menu and class arrays, and the
;; control class and title arrays, if the first element is FFFFh,
;; then the second element contains the ordinal value of a
;; resource or a predefined class, and the array contains no other
;; elements. For the dialog menu, class, and title arrays, and the
;; control class and title arrays, if the first element is any
;; value other than zero or FFFFh, then the array is assumed to be
;; a null-terminated Unicode string. Depending on the array, this
;; Unicode string can specify the name of a menu resource, a
;; registered class, the dialog title, or the initial text for a
;; control. For the control creation data array, if the first
;; element is non-zero, then it contains the length, in bytes, of
;; the creation data that follows. The fourth array following the
;; DLGTEMPLATE structure, which is recognized only when the dialog
;; style includes DS_SETFONT, specifies the font point size value
;; in the first element, followed by the name of the typeface as
;; a null-terminated Unicode string.
;;
;; Note that this implementation automatically sets the DS_SETFONT
;; style when the typeface and point size are specified.
;;
;; This implementation does not permit a menu or class specification
;; for the dialog, or creation data for the controls.
;;
;; The DLGTEMPLATE and DLGITEMTEMPLATE structures must be aligned
;; on a DWORD boundary. The variable-length arrays that follow the
;; structures must start on a WORD boundary, but with WORD-size
;; elements and the structure sizes and alignment requirements,
;; this should be automatic.
;=========================================================================

;----------------------------------------------------------------------
; The DLGTEMPLATE structure is declared incorrectly in windows.inc (or
; at least was at the time that this source was created), causing the
; structure size to be 20 bytes instead of the 18 bytes that the
; Windows API expects. The corrected declaration below has been renamed
; so it will not conflict with the declaration in windows.inc.
;----------------------------------------------------------------------

_DLGTEMPLATE STRUCT
  style             DWORD ?
  dwExtendedStyle   DWORD ?
  cdit              WORD  ?
  x                 WORD  ?
  y                 WORD  ?
  lx                WORD  ?
  ly                WORD  ?
_DLGTEMPLATE ENDS

;=========================================================================

.data

    ;---------------------------------------------------------------
    ; These module-level variables are used to simplify the code by
    ; reducing the number of parameters that must be passed to the
    ; procedures. The first (LongPointerDialogTemplate) is used to
    ; store the starting address of the template memory. The second
    ; (LongPointerWord) is used by the DefineDialog and control-
    ; definition procedures to store the current address in the
    ; template memory, and used by the CreateModalDialog and
    ; CreateModelessDialog procedures to resize the memory block
    ; before the dialog is created. For each item that is added to
    ; the template this variable must be adjusted to point to the
    ; next WORD following the item.
    ;
    ; Because these variables are shared by the procedures, dialog
    ; definitions cannot be interleaved.
    ;---------------------------------------------------------------

    lpdt  dd 0
    lpw   dd 0

.code

;=========================================================================

;-------------------------------------------------------------------
; This procedure generates a Unicode string starting at the address
; specified by lpw, and then adjusts lpw so it points to the next
; WORD following the string and null terminator. For compatibility
; with Windows 9x the MuliByteToWideChar function is used to
; generate the strings.
;-------------------------------------------------------------------

GenUstring proc lpAsciiString:DWORD

    ;---------------------------------------------------
    ; If lpAsciiString is null, skip the first element
    ; of the array, leaving it set to zero (no string).
    ;---------------------------------------------------

    .IF lpAsciiString == 0

      add lpw, 2

    .ELSE

      ;-----------------------------------------------
      ; The string length must be adjusted to include
      ; the null terminator (see below).
      ;-----------------------------------------------

      invoke szLen, lpAsciiString
      add eax, 1

      ;------------------------------------------------------------
      ; CP_ACP specifies that the function should use the current
      ; system ANSI code page to perform the conversion to Unicode.
      ; The -1 value for the cbMultiByte parameter specifies that
      ; the string is null terminated, and causes the function to
      ; process the entire string including the null terminator.
      ;------------------------------------------------------------

      invoke MultiByteToWideChar, CP_ACP, MB_PRECOMPOSED,
                                  lpAsciiString, -1, lpw, eax

      ;---------------------------------------------------------
      ; MultiByteToWideChar returns the number of wide (16-bit)
      ; characters written to the buffer, so to properly update
      ; the pointer the length must first be doubled.
      ;---------------------------------------------------------

      add eax, eax
      add lpw, eax

    .ENDIF

    ret

GenUstring endp

;=========================================================================

;--------------------------------------------------------------------
; This procedure creates a modal dialog from the dialog box template
; pointed to by lpdt. Parameter hParent should be zero if the
; dialog is the main window of the application. Parameter dwInitParam
; specifies a value that is passed to the dialog box procedure in the
; lParam parameter of the WM_INITDIALOG message.
;
; The DialogBoxIndirectParam function does not return until the
; dialog box is destroyed with the EndDialog function, and when it
; does it returns whatever was specified in the nResult parameter
; of the function. After freeing the allocated memory this procedure
; returns whatever the DialogBoxIndirectParam function returned.
;--------------------------------------------------------------------

CreateModalDialog proc hParent:DWORD,
                       lpDialogProc:DWORD,
                       dwInitParam:DWORD

    ;----------------------------------------------
    ; Resize the memory block to fit the template.
    ;----------------------------------------------

    mov eax, lpw
    sub eax, lpdt
    invoke GlobalReAlloc, lpdt, eax, 0

    ;--------------------
    ; Create the dialog.
    ;--------------------

    invoke GetModuleHandle, 0
    invoke DialogBoxIndirectParam, eax, lpdt, hParent,
                                   lpDialogProc, dwInitParam

    ;----------------------------
    ; Free the allocated memory.
    ;----------------------------

    push eax
    invoke GlobalFree, lpdt
    pop eax

    ret

CreateModalDialog endp

;=========================================================================

;----------------------------------------------------------------
; This procedure creates a modeless dialog from the dialog box
; template pointed to by lpdt. Parameter hParent should be zero
; if the dialog is the main window of the application. Parameter
; lParamInit specifies a value that is passed to the dialog box
; procedure in the lParam parameter of the WM_INITDIALOG message.
;
; After freeing the allocated memory, this proc returns whatever
; the CreateDialogIndirectParam function returned, which in the
; absence of an error would be the handle to the dialog window.
;----------------------------------------------------------------

CreateModelessDialog proc hParent:DWORD,
                          lpDialogProc:DWORD,
                          lParamInit:DWORD

    ;----------------------------------------------
    ; Resize the memory block to fit the template.
    ;----------------------------------------------

    mov eax, lpw
    sub eax, lpdt
    invoke GlobalReAlloc, lpdt, eax , 0

    ;--------------------
    ; Create the dialog.
    ;--------------------

    invoke GetModuleHandle, 0
    invoke CreateDialogIndirectParam, eax, lpdt, hParent,
                                      lpDialogProc, lParamInit
    ;----------------------------
    ; Free the allocated memory.
    ;----------------------------

    push eax
    invoke GlobalFree, lpdt
    pop eax

    ret

CreateModelessDialog endp

;=========================================================================

;-----------------------------------------------------------------
; This procedure allocates memory for a dialog box template, then
; initializes the essential members of the DLGTEMPLATE structure
; and the menu, class, and title arrays, and optionally the font
; point size and typeface arrays, and adjusts lpw so it points
; to the next WORD following the last array.
;
; The initial allocation size, 10KB, should be sufficient even
; for a large and complex dialog. In the unlikely event that the
; allocation were not sufficient, the likely result would be a
; memory access violation at run time.
;
; The CreateModalDialog and CreateModelessDialog procedures resize
; the memory block to fit the template prior to creating the
; dialog, and then free the remaining memory when the template is
; no longer needed.
;
; Parameter ctlcnt must match the number of controls defined.
; If the value is too high then the function that creates the
; dialog will fail. If the value is too low then one or more
; of the controls will not be created.
;-----------------------------------------------------------------

DefineDialog proc lpszTitle:DWORD,
                  style:DWORD,
                  tx:DWORD,
                  ty:DWORD,
                  wd:DWORD,
                  ht:DWORD,
                  lpszTypeFace:DWORD,
                  pointSize:DWORD,
                  ctlcnt:DWORD

    ;-----------------------------------------------------------
    ; Allocate memory to store the dialog box template and copy
    ; the returned pointer to lpdt. Note that the code below
    ; depends on the memory being initialized to zero. Note also
    ; that memory allocated by GlobalAlloc is guaranteed to be
    ; aligned on an 8-byte boundary.
    ;-----------------------------------------------------------

    invoke GlobalAlloc, GMEM_FIXED or GMEM_ZEROINIT, 1024*10
    mov lpdt, eax

    ;----------------------------------------------------
    ; Initialize the essential members of the structure.
    ;----------------------------------------------------

    mov edx, style
    mov [eax]._DLGTEMPLATE.style, edx
    mov edx, ctlcnt
    mov [eax]._DLGTEMPLATE.cdit, dx
    mov edx, tx
    mov [eax]._DLGTEMPLATE.x, dx
    mov edx, ty
    mov [eax]._DLGTEMPLATE.y, dx
    mov edx, wd
    mov [eax]._DLGTEMPLATE.lx, dx
    mov edx, ht
    mov [eax]._DLGTEMPLATE.ly, dx

    ;---------------------------------------------
    ; Initialize lpw and adjust it so it points
    ; to the next WORD following the structure.
    ;---------------------------------------------

    mov lpw, eax
    add lpw, SIZEOF _DLGTEMPLATE

    ;-------------------------------------------
    ; Skip the first element of the menu array,
    ; leaving it set to zero (no menu).
    ;-------------------------------------------

    add lpw, 2

    ;------------------------------------------------------
    ; Skip the first element of the class array, leaving it
    ; set to zero (no class), so the system will use the
    ; predefined dialog box class.
    ;------------------------------------------------------

    add lpw, 2

    ;---------------------------------------------------
    ; Initialize the title array and adjust lpw so it
    ; points to the next WORD following the title array.
    ;---------------------------------------------------

    invoke GenUstring, lpszTitle

    ;-----------------------------------------------------
    ; If lpszTypeFace and pointSize are both non-zero, set
    ; the DS_SETFONT style, initialize the font point size
    ; and typeface arrays, and adjust lpw so it points
    ; to the next WORD following the typeface array.
    ;-----------------------------------------------------

    cmp pointSize, 0
    je  @F
    cmp lpszTypeFace, 0
    je  @F
    mov eax, lpdt
    or [eax]._DLGTEMPLATE.style, DS_SETFONT
    mov eax, lpw
    mov edx, pointSize
    mov [eax], dx
    add lpw, 2
    invoke GenUstring, lpszTypeFace

  @@:

    ret

DefineDialog endp

;=========================================================================

;------------------------------------------------------------------
; Starting at the address specified by lpw, this general-purpose
; control definition procedure initializes the essential members
; of a DLGITEMTEMPLATE structure and the class, title and creation
; data arrays.
;
; Parameter lpszClass should specify the control class, typically
; as one of the strings listed below.
;
; "Button"
; "ComboBox"
; "Edit"
; "ListBox"
; "ScrollBar"
; "Static"
;
; "ComboBoxEx32"
; "commctrl_DragListMsg"
; "msctls_hotkey32"
; "msctls_progress32"
; "msctls_statusbar32"
; "msctls_trackbar32"
; "msctls_updown32"
; "NativeFontCtl"
; "ReBarWindow32"
; "RichEdit"
; "RichEdit20a"
; "SysAnimate32"
; "SysDateTimePick32"
; "SysHeader32"
; "SysIPAddress32"
; "SysListView32"
; "SysMonthCal32"
; "SysPager"
; "SysTabControl32"
; "SysTreeView32"
; "ToolbarWindow32"
; "tooltips_class32"
;
; The title array can specify the caption or initial text for the
; control, or the ordinal value of a resource in the executable
; file. Specify the caption or initial text in the lpszTitle
; parameter or an ordinal value in the rid parameter. If the rid
; parameter is non-zero then the lpszTitle parameter is ignored.
;
; There is no support for creation data.
;
; For Windows 95/98/ME, only the low-order byte of the control
; id (parameter cid) is used, so the maximum value is 255.
;
; The tab order of the controls in a dialog is determined by the
; order in which the controls are created, and by which controls
; have the WS_TABSTOP style.
;
; Note that for maximum flexibility this procedure does not
; specify the WS_VISIBLE or WS_CHILD styles that would normally
; be required for a control in a dialog.
;------------------------------------------------------------------

DefineControl proc lpszClass:DWORD,
                   lpszTitle:DWORD,
                   style:DWORD,
                   tx:DWORD,
                   ty:DWORD,
                   wd:DWORD,
                   ht:DWORD,
                   rid:DWORD,
                   cid:DWORD

    ;--------------------------------------------------------------------
    ; The DLGITEMTEMPLATE structure must be aligned on a DWORD boundary.
    ; This is accomplished by first adding the maximum value necessary
    ; to extend the address to a dword alignment, and then clearing the
    ; lower two bits.
    ;--------------------------------------------------------------------

    add lpw, 3
    and lpw, NOT 3

    ;----------------------------------------------------
    ; Initialize the essential members of the structure.
    ;----------------------------------------------------

    mov eax, lpw
    mov edx, style
    mov [eax].DLGITEMTEMPLATE.style, edx
    mov edx, tx
    mov [eax].DLGITEMTEMPLATE.x, dx
    mov edx, ty
    mov [eax].DLGITEMTEMPLATE.y, dx
    mov edx, wd
    mov [eax].DLGITEMTEMPLATE.lx, dx
    mov edx, ht
    mov [eax].DLGITEMTEMPLATE.cy, dx
    mov edx, cid
    mov [eax].DLGITEMTEMPLATE.id, dx

    ;---------------------------------------
    ; Adjust lpw so it points to the next
    ; WORD following the structure.
    ;---------------------------------------

    add lpw, SIZEOF DLGITEMTEMPLATE

    ;----------------------------------------------------
    ; Initialize the class array and adjust lpw so it
    ; points to the next WORD following the class array.
    ;----------------------------------------------------

    invoke GenUstring, lpszClass

    ;------------------------------------------------------
    ; Initialize the title array to the caption or initial
    ; text specified by lpszTitle, or to the resource ID
    ; specified by rid, and adjust lpw so it points to
    ; the next WORD following the title array.
    ;------------------------------------------------------

    .IF rid
      mov eax, lpw
      mov edx, 0ffffh
      mov [eax], dx
      mov edx, rid
      mov [eax+2], dx
      add lpw, 4
    .ELSE
      invoke GenUstring, lpszTitle
    .ENDIF

    ;----------------------------------------------------
    ; Skip the first element of the creation data array,
    ; leaving it set to zero (no creation data).
    ;----------------------------------------------------

    add lpw, 2

    ret

DefineControl endp

;=========================================================================
; The following specialized control-definition procedures are
; simply wrappers for the general-purpose procedure.
;=========================================================================

DefinePushButton proc lpszCaption:DWORD,
                      style:DWORD,
                      tx:DWORD,
                      ty:DWORD,
                      wd:DWORD,
                      ht:DWORD,
                      cid:DWORD

    mov eax, BS_PUSHBUTTON or WS_VISIBLE or WS_CHILD
    or eax, style

    invoke DefineControl, chr$("Button"), lpszCaption, eax,
                          tx, ty, wd, ht,
                          0, cid
    ret

DefinePushButton endp

;=========================================================================

DefineDefPushButton proc lpszCaption:DWORD,
                         style:DWORD,
                         tx:DWORD,
                         ty:DWORD,
                         wd:DWORD,
                         ht:DWORD,
                         cid:DWORD

    mov eax, BS_DEFPUSHBUTTON or WS_VISIBLE or WS_CHILD
    or eax, style

    invoke DefineControl, chr$("Button"), lpszCaption, eax,
                          tx, ty, wd, ht,
                          0,  cid
    ret

DefineDefPushButton endp

;=========================================================================

DefineAutoCheckBox proc lpszCaption:DWORD,
                        style:DWORD,
                        tx:DWORD,
                        ty:DWORD,
                        wd:DWORD,
                        ht:DWORD,
                        cid:DWORD

    mov eax, BS_AUTOCHECKBOX or WS_VISIBLE or WS_CHILD
    or eax, style

    invoke DefineControl, chr$("Button"), lpszCaption, eax,
                          tx, ty, wd, ht,
                          0, cid
    ret

DefineAutoCheckBox endp

;=========================================================================

DefineAutoRadioButton proc lpszCaption:DWORD,
                           style:DWORD,
                           tx:DWORD,
                           ty:DWORD,
                           wd:DWORD,
                           ht:DWORD,
                           cid:DWORD

    mov eax, BS_AUTORADIOBUTTON or WS_VISIBLE or WS_CHILD
    or eax, style

    invoke DefineControl, chr$("Button"), lpszCaption, eax,
                          tx, ty, wd, ht,
                          0, cid
    ret

DefineAutoRadioButton endp

;=========================================================================

DefineGroupBox proc lpszCaption:DWORD,
                    style:DWORD,
                    tx:DWORD,
                    ty:DWORD,
                    wd:DWORD,
                    ht:DWORD,
                    cid:DWORD

    mov eax, BS_GROUPBOX or WS_VISIBLE or WS_CHILD
    or eax, style

    invoke DefineControl, chr$("Button"), lpszCaption, eax,
                          tx, ty, wd, ht,
                          0, cid
    ret

DefineGroupBox endp

;=========================================================================

DefineEditText proc lpszText:DWORD,
                    style:DWORD,
                    tx:DWORD,
                    ty:DWORD,
                    wd:DWORD,
                    ht:DWORD,
                    cid:DWORD

    mov eax, WS_VISIBLE or WS_CHILD
    or eax, style

    invoke DefineControl, chr$("Edit"), lpszText, eax,
                          tx, ty, wd, ht,
                          0, cid
    ret

DefineEditText endp

;=========================================================================

DefineLText proc lpszText:DWORD,
                 style:DWORD,
                 tx:DWORD,
                 ty:DWORD,
                 wd:DWORD,
                 ht:DWORD,
                 cid:DWORD

    mov eax, SS_LEFT or WS_VISIBLE or WS_CHILD
    or eax, style

    invoke DefineControl, chr$("Static"), lpszText, eax,
                          tx, ty, wd, ht,
                          0, cid
    ret

DefineLText endp

;=========================================================================

DefineRText proc lpszText:DWORD,
                 style:DWORD,
                 tx:DWORD,
                 ty:DWORD,
                 wd:DWORD,
                 ht:DWORD,
                 cid:DWORD

    mov eax, SS_RIGHT or WS_VISIBLE or WS_CHILD
    or eax, style

    invoke DefineControl, chr$("Static"), lpszText, eax,
                          tx, ty, wd, ht,
                          0, cid
    ret

DefineRText endp

;=========================================================================

DefineCText proc lpszText:DWORD,
                 style:DWORD,
                 tx:DWORD,
                 ty:DWORD,
                 wd:DWORD,
                 ht:DWORD,
                 cid:DWORD

    mov eax, SS_CENTER or WS_VISIBLE or WS_CHILD
    or eax, style

    invoke DefineControl, chr$("Static"), lpszText, eax,
                          tx, ty, wd, ht,
                          0, cid
    ret

DefineCText endp

;=========================================================================

DefineBitmap proc style:DWORD, tx:DWORD, ty:DWORD, rid:DWORD, cid:DWORD

    ;----------------------------------------------------------
    ; For the SS_BITMAP style the width and height are ignored
    ; and the control is sized to fit the bitmap.
    ;----------------------------------------------------------

    mov eax, SS_BITMAP or WS_VISIBLE or WS_CHILD
    or eax, style

    invoke DefineControl, chr$("Static"), 0, eax,
                          tx, ty, 0, 0,
                          rid, cid
    ret

DefineBitmap endp

;=========================================================================

DefineIcon proc style:DWORD, tx:DWORD, ty:DWORD, rid:DWORD, cid:DWORD

    ;--------------------------------------------------------
    ; For the SS_ICON style the width and height are ignored
    ; and the control is sized to fit the icon.
    ;--------------------------------------------------------

    mov eax, SS_ICON or WS_VISIBLE or WS_CHILD
    or eax, style

    invoke DefineControl, chr$("Static"), 0, eax,
                          tx, ty, 0, 0,
                          rid, cid
    ret

DefineIcon endp

;=========================================================================

DefineListBox proc style:DWORD,
                   tx:DWORD,
                   ty:DWORD,
                   wd:DWORD,
                   ht:DWORD,
                   cid:DWORD

    mov eax, WS_VISIBLE or WS_CHILD
    or eax, style

    invoke DefineControl, chr$("ListBox"), 0, eax,
                          tx, ty, wd, ht,
                          0, cid
    ret

DefineListBox endp

;=========================================================================

DefineComboBox proc style:DWORD,
                    tx:DWORD,
                    ty:DWORD,
                    wd:DWORD,
                    ht:DWORD,
                    cid:DWORD

    mov eax, WS_VISIBLE or WS_CHILD
    or eax, style

    invoke DefineControl, chr$("ComboBox"), 0, eax,
                          tx, ty, wd, ht,
                          0, cid
    ret

DefineComboBox endp

;=========================================================================

DefineScrollBar proc style:DWORD,
                     tx:DWORD,
                     ty:DWORD,
                     wd:DWORD,
                     ht:DWORD,
                     cid:DWORD

    mov eax, WS_VISIBLE or WS_CHILD
    or eax, style

    invoke DefineControl, chr$("ScrollBar"), 0, eax,
                          tx, ty, wd, ht,
                          0, cid
    ret

DefineScrollBar endp

;=========================================================================

;-----------------------------------------------------------------
; To use a Rich Edit control your app must first call LoadLibrary
; to load the appropriate DLL - RICHED20.DLL for version 2 or 3,
; or RICHED32.DLL for version 1.
;-----------------------------------------------------------------

;---------------------------------------------
; This procedure is coded for version 2 or 3.
;---------------------------------------------

DefineRichEdit proc style:DWORD,
                    tx:DWORD,
                    ty:DWORD,
                    wd:DWORD,
                    ht:DWORD,
                    cid:DWORD

    mov eax, WS_VISIBLE or WS_CHILD
    or eax, style

    invoke DefineControl, chr$("RichEdit20a"), 0, eax,
                          tx, ty, wd, ht,
                          0, cid
    ret

DefineRichEdit endp

;----------------------------------------
; This procedure is coded for version 1.
;----------------------------------------

DefineRichEdit1 proc style:DWORD,
                     tx:DWORD,
                     ty:DWORD,
                     wd:DWORD,
                     ht:DWORD,
                     cid:DWORD

    mov eax, WS_VISIBLE or WS_CHILD
    or eax, style

    invoke DefineControl, chr$("RichEdit"), 0, eax,
                          tx, ty, wd, ht,
                          0, cid
    ret

DefineRichEdit1 endp

;=========================================================================

;---------------------------------------------------------------------
; To use controls from the common control DLL specific common control
; classes must first be initialized. This procedure initializes 14 of
; the commonly used common control classes in a single call.
;
; This procedure uses the InitCommonControlsEx function, which should
; work for all recent versions of Windows, but it may be necessary to
; substitute the older InitCommonControls function for Windows 95 or
; NT.
;
; Note that ICC_WIN95_CLASSES includes the following:
;
; ICC_LISTVIEW_CLASSES listview, header
; ICC_TREEVIEW_CLASSES treeview, tooltips
; ICC_BAR_CLASSES      toolbar, statusbar, trackbar, tooltips
; ICC_TAB_CLASSES      tab, tooltips
; ICC_UPDOWN_CLASS     updown
; ICC_PROGRESS_CLASS   progress
; ICC_HOTKEY_CLASS     hotkey
; ICC_ANIMATE_CLASS    animate
;
; And here is a small amount of imformation on the other classes:
;
; ICC_DATE_CLASSES     month picker, date picker, time picker, updown
; ICC_USEREX_CLASSES   comboex
; ICC_COOL_CLASSES     rebar
; ICC_INTERNET_CLASSES    IP address
; ICC_PAGESCROLLER_CLASS  pager
; ICC_NATIVEFNTCTL_CLASS  native font control
;---------------------------------------------------------------------

InitializeCommonControls proc

    LOCAL icce:INITCOMMONCONTROLSEX

    ;-----------------------------------------------------
    ; Set the dwSize member to the size of the structure.
    ;-----------------------------------------------------

    mov icce.dwSize, SIZEOF INITCOMMONCONTROLSEX

    ;--------------------------------------------------------------
    ; Set the bit flags to indicate which common controls to load.
    ;--------------------------------------------------------------

    mov icce.dwICC, ICC_WIN95_CLASSES or \
                    ICC_DATE_CLASSES or \
                    ICC_USEREX_CLASSES or \
                    ICC_COOL_CLASSES or \
                    ICC_INTERNET_CLASSES or \
                    ICC_PAGESCROLLER_CLASS or \
                    ICC_NATIVEFNTCTL_CLASS

    invoke InitCommonControlsEx, ADDR icce

    ret

InitializeCommonControls endp

;=========================================================================