;=========================================================================
; This file is a collection of support procedures. Note that like the
; demos, there is no significant error checking or handling.
;=========================================================================

;--------------------------------------------------------------------
; Dialogs were no typeface or point size is specified in the dialog
; template use the system font. For dialogs that use the system font,
; the GetDialogBaseUnits function returns the horizontal and vertical
; dialog base units that the system uses to convert dialog template
; units (DTUs) to pixels. See README.TXT for more information.
;--------------------------------------------------------------------

;------------------------------------------------------------
; The following procedures convert from pixel units to DTUs.
;
; These procedures are called from the pdx and pdy macro
; functions, so to avoid unexpected changes in the register
; values they preserve all registers other than eax.
;
; Note that these procedures will work correctly only if the
; dialog uses the system font, and that the conversions may
; not produce the same, exact result under all versions of
; Windows. See the Microsoft GetDialogBaseUnits documentation
; for the conversion details.
;------------------------------------------------------------

PixelToDtuX proc pixelX:DWORD
    push ecx
    push edx
    invoke GetDialogBaseUnits
    mov ecx, loword(eax)
    mov eax, pixelX
    shl eax, 2
    cdq
    div ecx
    pop edx
    pop ecx
    ret
PixelToDtuX endp

PixelToDtuY proc pixelY:DWORD
    push ecx
    push edx
    invoke GetDialogBaseUnits
    mov ecx, hiword(eax)
    mov eax, pixelY
    shl eax, 3
    cdq
    div ecx
    pop edx
    pop ecx
    ret
PixelToDtuY endp

;-------------------------------------------------------------------
; The following procedures return the width or height, in DTUs, for
; a dialog with a client area of the specified width or height, in
; pixels. The pixel width must be adjusted to allow for the vertical
; borders and the pixel height to allow for the horizontal borders
; and the title bar. The 2-pixel adjustment on the pixel height
; corrects for the excess height of the title bar over the value of
; the SM_CYSIZE metric.
;
; Note that these procedures will work correctly only if the dialog
; uses the system font, and that the returned width may not produce
; a dialog with a client area of exactly the specified pixel size
; under all versions of Windows.
;-------------------------------------------------------------------

ClientPixelWidthToDialogWidth proc clientPixelWidth:DWORD
    invoke GetSystemMetrics, SM_CXFIXEDFRAME
    add eax, eax
    add eax, clientPixelWidth
    invoke PixelToDtuX, eax
    ret
ClientPixelWidthToDialogWidth endp

ClientPixelHeightToDialogHeight proc clientPixelHeight:DWORD
    invoke GetSystemMetrics, SM_CYSIZE
    push eax
    invoke GetSystemMetrics, SM_CYFIXEDFRAME
    add eax, eax
    pop ecx
    add eax, ecx
    add eax, clientPixelHeight
    add eax, 2
    invoke PixelToDtuY, eax
    ret
ClientPixelHeightToDialogHeight endp

;=========================================================================

;---------------------------------------------------------
; This procedure sizes the specified window so the client
; area is exactly the specified width and height.
;---------------------------------------------------------

SetClientSize proc uses ebx hwnd:HWND, pixelWidth:DWORD, pixelHeight:DWORD

    LOCAL rcc:RECT, rcw:RECT

    invoke GetClientRect, hwnd, ADDR rcc
    invoke GetWindowRect, hwnd, ADDR rcw

    mov ecx, rcw.right
    sub ecx, rcw.left       ; ecx = window width - 1

    mov eax, pixelWidth
    dec eax                 ; eax = pixelWidth - 1
    mov ebx, rcc.right      ; ebx = client width - 1
    sub ebx, eax            ; ebx = difference
    sub ecx, ebx            ; adjust width

    mov edx, rcw.bottom     ; edx = window height - 1
    sub edx, rcw.top

    mov eax, pixelHeight
    dec eax                 ; eax = pixelHeight - 1
    mov ebx, rcc.bottom     ; ebx = client height - 1
    sub ebx, eax            ; ebx = difference
    sub edx, ebx            ; adjust height

    invoke MoveWindow, hwnd, rcw.left, rcw.top, ecx, edx, TRUE

    ret

SetClientSize endp

;=========================================================================

DrawFilledCircle proc uses ebx hdc:HDC,
                               center_x:DWORD,
                               center_y:DWORD,
                               radius:DWORD,
                               color:COLORREF

    LOCAL hPen:HPEN, hBrush:HBRUSH

    ;----------------------------------------------------------------
    ; Create a pen and a brush of the required color and select them
    ; into the specified DC. The pen will create a solid line with a
    ; width of one pixel.
    ;
    ; SelectObject returns the previously selected object. Since we
    ; dont control the DC, we need to replace the new object with
    ; the original after we finish drawing with it.
    ;----------------------------------------------------------------

    invoke CreatePen, PS_SOLID, 1, color
    mov hPen, eax
    invoke SelectObject, hdc, hPen
    push eax
    invoke CreateSolidBrush, color
    mov hBrush, eax
    invoke SelectObject, hdc, hBrush
    push eax

    ;------------------------------------------------------
    ; Calculate the coordinates of the bounding rectangle.
    ;------------------------------------------------------

    mov eax, center_x
    sub eax, radius
    mov ebx, center_y
    sub ebx, radius
    mov ecx, center_x
    add ecx, radius
    mov edx, center_y
    add edx, radius

    ;------------------
    ; Draw the circle.
    ;------------------

    invoke Ellipse, hdc, eax, ebx, ecx, edx

    ;-----------------------------------------------
    ; Select the original objects back into the DC.
    ;-----------------------------------------------

    pop eax
    invoke SelectObject, hdc, eax
    pop eax
    invoke SelectObject, hdc, eax

    ;-------------------------------------------------------
    ; Free the resources associated with the pen and brush.
    ;-------------------------------------------------------

    invoke DeleteObject, hPen
    invoke DeleteObject, hBrush

    ret

DrawFilledCircle endp

;=========================================================================

;------------------------------------------------------------------
; This procedure creates a DIB from a DDB and stores it in a file,
; or silently fails if the bits per pixel (bpp) for the DDB is not
; 4, 8, 16, 24, or 32, or if the bpp value for the DIB, specified
; in bppDib, is not 4, 8, or 24.
;------------------------------------------------------------------

CreateBitmapFile proc uses ebx edi hbmp:HBITMAP,
                                   bppDib:DWORD,
                                   pszFile:DWORD

    LOCAL dwtmp:DWORD
    LOCAL hFile:HANDLE
    LOCAL hHeap:HANDLE
    LOCAL hdcScreen:HDC
    LOCAL hdr:BITMAPFILEHEADER

    invoke GetProcessHeap
    mov hHeap, eax

    invoke GetDC, NULL
    mov hdcScreen, eax

    ;---------------------------------------------
    ; Allocate memory for a BITMAPINFO structure.
    ;---------------------------------------------

    invoke HeapAlloc, hHeap, HEAP_ZERO_MEMORY, SIZEOF BITMAPINFO
    mov ebx, eax

    ;-----------------------------------------------
    ; Set the biSize member of the BITMAPFILEHEADER
    ; structure to the size of the structure.
    ;-----------------------------------------------

    mov [ebx].BITMAPINFO.bmiHeader.biSize, SIZEOF BITMAPINFOHEADER

    ;--------------------------------------------------------
    ; Call the GetDIBits function with the lpvBits parameter
    ; set to NULL, so the function will pass the dimensions
    ; and format of the bitmap to the BITMAPINFO structure.
    ;--------------------------------------------------------

    invoke GetDIBits, hdcScreen, hbmp, 0, 0, NULL,
                      ebx, DIB_RGB_COLORS

    ;--------------------------------------------------
    ; Fail if the input bpp is not 4, 8, 16, 24 or 32.
    ;--------------------------------------------------

    movzx eax, [ebx].BITMAPINFO.bmiHeader.biBitCount
    SWITCH eax
      CASE 4, 8, 16, 24, 32
      DEFAULT
        invoke HeapFree, hHeap, 0, ebx
        ret
    ENDSW

    ;------------------------------------------------------------
    ; Set the output bpp, or fail if the bpp is not 4, 8, or 24.
    ;------------------------------------------------------------

    mov eax, bppDib
    SWITCH eax
      CASE 4, 8, 24
        mov [ebx].BITMAPINFO.bmiHeader.biBitCount, ax
      DEFAULT
        invoke HeapFree, hHeap, 0, ebx
        ret
    ENDSW

    ;--------------------------------------------------------------
    ; For 4 or 8 bpp, reallocate the memory to allow space for the
    ; color table (RGBQUAD array).
    ;
    ; The HeapReAlloc function, when reallocating a memory block
    ; to a larger size, even with the HEAP_ZERO_MEMORY flag, will
    ; preserve the contents of the original block.
    ;--------------------------------------------------------------

    movzx eax, [ebx].BITMAPINFO.bmiHeader.biBitCount
    SWITCH eax
      CASE 4
        invoke HeapReAlloc, hHeap, HEAP_ZERO_MEMORY, ebx,
                            SIZEOF BITMAPINFO + 16 * SIZEOF RGBQUAD
        mov ebx, eax
      CASE 8
        invoke HeapReAlloc, hHeap, HEAP_ZERO_MEMORY, ebx,
                            SIZEOF BITMAPINFO + 256 * SIZEOF RGBQUAD
        mov ebx, eax
    ENDSW

    ;-------------------------------------------------------------------
    ; Assume the bitmap will not be compressed and set the BI_RGB flag.
    ;-------------------------------------------------------------------

    mov [ebx].BITMAPINFO.bmiHeader.biCompression, BI_RGB

    ;-------------------------------------------------------------
    ; Calc the number of bytes for the bitmap image and store the
    ; result in the biSizeImage member. The scan lines must be
    ; DWORD aligned, so the width in bits must be adjusted to a
    ; multiple of 32 by adding 31 and then clearing the lower 5
    ; bits.
    ;-------------------------------------------------------------

    movzx edi, [ebx].BITMAPINFO.bmiHeader.biBitCount
    mov eax, [ebx].BITMAPINFO.bmiHeader.biWidth
    mul edi
    add eax, 31
    and eax, NOT 31
    shr eax, 3
    mov ecx, [ebx].BITMAPINFO.bmiHeader.biHeight
    mul ecx
    mov [ebx].BITMAPINFO.bmiHeader.biSizeImage, eax

    ;---------------------------------------------------------------
    ; Calc the horizontal and vertical resolution for the screen in
    ; pixels per meter and store the results in the biXPelsPerMeter
    ; and biYPelsPerMeter members. For index values LOGPIXELSX and
    ; LOGPIXELSY the GetDeviceCaps function returns the resolution
    ; in pixels per logical inch.
    ;---------------------------------------------------------------

    invoke GetDeviceCaps, hdcScreen, LOGPIXELSX
    invoke MulDiv, eax, 39370, 1000
    mov [ebx].BITMAPINFO.bmiHeader.biXPelsPerMeter, eax

    invoke GetDeviceCaps, hdcScreen, LOGPIXELSY
    invoke MulDiv, eax, 39370, 1000
    mov [ebx].BITMAPINFO.bmiHeader.biYPelsPerMeter, eax

    ;------------------------------------------------
    ; For 4 or 8 bpp set the biClrUsed member to the
    ; correct value, otherwise leave it set to 0.
    ;------------------------------------------------

    movzx eax, [ebx].BITMAPINFO.bmiHeader.biBitCount
    .IF eax == 4
      mov [ebx].BITMAPINFO.bmiHeader.biClrUsed, 16
    .ELSEIF eax == 8
      mov [ebx].BITMAPINFO.bmiHeader.biClrUsed, 256
    .ENDIF

    ;------------------------------------------------
    ; Set the biClrImportant member to 0, indicating
    ; that all of the device colors are important.
    ;------------------------------------------------

    mov [ebx].BITMAPINFO.bmiHeader.biClrImportant, 0

    ;-----------------------------------------------
    ; Allocate a buffer to receive the bitmap data.
    ;-----------------------------------------------

    invoke HeapAlloc, hHeap, HEAP_ZERO_MEMORY,
                      [ebx].BITMAPINFO.bmiHeader.biSizeImage
    mov edi, eax

    ;--------------------------------------------------------------
    ; Call the GetDIBits function to retrieve the bitmap bits from
    ; the DDB and store them, and the synthesized color table, if
    ; present, in the buffer.
    ;
    ; The biClrUsed member must be preserved around the call to
    ; prevent the function from setting it to 0.
    ;--------------------------------------------------------------

    push [ebx].BITMAPINFO.bmiHeader.biClrUsed
    invoke GetDIBits, hdcScreen,
                      hbmp,
                      0,
                      [ebx].BITMAPINFO.bmiHeader.biHeight,
                      edi,
                      ebx,
                      DIB_RGB_COLORS
    pop [ebx].BITMAPINFO.bmiHeader.biClrUsed

    ;------------------
    ; Create the file.
    ;------------------

    invoke CreateFile, pszFile,
                       GENERIC_READ or GENERIC_WRITE,
                       0,
                       NULL,
                       CREATE_ALWAYS,
                       FILE_ATTRIBUTE_NORMAL,
                       NULL
    mov hFile, eax

    ;---------------------------------------------------
    ; Calculate the size of the file and initialize the
    ; members of the BITMAPFILEHEADER.
    ;---------------------------------------------------

    mov eax, [ebx].BITMAPINFO.bmiHeader.biClrUsed
    mov ecx, SIZEOF RGBQUAD
    mul ecx
    add eax, SIZEOF BITMAPFILEHEADER
    add eax, [ebx].BITMAPINFO.bmiHeader.biSize
    push eax
    add eax, [ebx].BITMAPINFO.bmiHeader.biSizeImage

    ;------------------------------------------------------
    ; The bfType member must be 'BM', as stored in memory.
    ;------------------------------------------------------

    mov hdr.bfType, 4d42h

    mov hdr.bfSize, eax
    mov hdr.bfReserved1, 0
    mov hdr.bfReserved2, 0
    pop eax
    mov hdr.bfOffBits, eax

    ;----------------------------------------------------
    ; Copy the BITMAPFILEHEADER structure into the file.
    ;----------------------------------------------------

    invoke WriteFile, hFile,
                      ADDR hdr,
                      SIZEOF BITMAPFILEHEADER,
                      ADDR dwtmp,
                      NULL

    ;-------------------------------------------------------------
    ; Copy the BITMAPINFOHEADER structure and the RGBQUAD array,
    ; if present, into the file. The bytes to write is passed in
    ; EDX instead of EAX because for this code INVOKE will use
    ; EAX as a scratch register, and MASM would otherwise return:
    ;   error A2133: register value overwritten by INVOKE
    ;-------------------------------------------------------------

    mov eax, [ebx].BITMAPINFO.bmiHeader.biClrUsed
    mov ecx, SIZEOF RGBQUAD
    mul ecx
    add eax, SIZEOF BITMAPINFOHEADER
    mov edx, eax

    invoke WriteFile, hFile,
                      ebx,
                      edx,
                      ADDR dwtmp,
                      NULL

    ;-------------------------------------
    ; Copy the bitmap bits into the file.
    ;-------------------------------------

    invoke WriteFile, hFile,
                      edi,
                      [ebx].BITMAPINFO.bmiHeader.biSizeImage,
                      ADDR dwtmp,
                      NULL

    ;-----------
    ; Clean up.
    ;-----------

    invoke ReleaseDC, NULL, hdcScreen
    invoke HeapFree, hHeap, 0, ebx
    invoke HeapFree, hHeap, 0, edi
    invoke CloseHandle, hFile

    ret

CreateBitmapFile endp
