;------------------------------------------------------------------
; These macro procedures simplify the task of loading strings into
; list boxes and combo boxes. The reparg macro allows these macros
; to accept quoted strings as well as string addresses.
;------------------------------------------------------------------

LBADDS MACRO h,s
    invoke SendMessage, h, LB_ADDSTRING, 0, reparg(s)
ENDM

CBADDS MACRO h,s
    invoke SendMessage, h, CB_ADDSTRING, 0, reparg(s)
ENDM

;--------------------------------------------------------------------
; This macro procedure is used to simplify the task of associating a
; bitmap image with an SS_BITMAP static control.
;--------------------------------------------------------------------

SETIMAGE MACRO hwnd, hbmp
    invoke SendMessage, hwnd, STM_SETIMAGE, IMAGE_BITMAP, hbmp
ENDM

;---------------------------------------------------------------
; This macro function is used to simplify the task of loading a
; bitmap image resource. The return value is the bitmap handle.
;---------------------------------------------------------------

loadimage MACRO rid
    invoke GetModuleHandle, NULL
    invoke LoadImage, eax, rid, IMAGE_BITMAP, 0, 0, 0
    EXITM <eax>
ENDM

;-----------------------------------------------------------
; This macro function returns a COLORREF value, calculated
; entirely by the preprocessor from the specified color
; component intensities, which have a valid range of 0-255.
;-----------------------------------------------------------

rgb MACRO red, green, blue
    EXITM % blue SHL 16 + green SHL 8 + red
ENDM

;-------------------------------------------------------------
; These macro functions simplify the tasks of extracting the
; the low and high words of a dword argument, as is commonly
; needed when processing WM_COMMAND and other messages.
;
; These macros avoid the non-obvious side effect of modifying
; a register by using dedicated memory variables. The oddball
; names reduce the possibility of name conflicts.
;-------------------------------------------------------------

loword MACRO arg
    IFNDEF __loword_macro_return_var__
      .data
        __loword_macro_return_var__ dd 0
      .code
    ENDIF
    push arg
    pop __loword_macro_return_var__
    and __loword_macro_return_var__, 0ffffh
    EXITM <__loword_macro_return_var__>
ENDM

hiword MACRO arg
    IFNDEF __hiword_macro_return_var__
      .data
        __hiword_macro_return_var__ dd 0
      .code
    ENDIF
    push arg
    pop __hiword_macro_return_var__
    shr __hiword_macro_return_var__, 16
    EXITM <__hiword_macro_return_var__>
ENDM

;--------------------------------------------------------------------
; These macro functions convert from pixel units to Dialog Template
; Units (DTUs).
;
; Note that these macros will work correctly only if the dialog uses
; the system font, and that the conversions may not produce the same,
; exact result under all versions of Windows.
;--------------------------------------------------------------------

pdx MACRO pixelX
    IFNDEF __pdx_macro_return_var__
      .data
        __pdx_macro_return_var__ dd 0
      .code
    ENDIF
    push eax
    invoke PixelToDtuX, pixelX
    mov __pdx_macro_return_var__, eax
    pop eax
    EXITM <__pdx_macro_return_var__>
ENDM

pdy MACRO pixelY
    IFNDEF __pdy_macro_return_var__
      .data
        __pdy_macro_return_var__ dd 0
      .code
    ENDIF
    push eax
    invoke PixelToDtuY, pixelY
    mov __pdy_macro_return_var__, eax
    pop eax
    EXITM <__pdy_macro_return_var__>
ENDM

;------------------------------------------------------------
; These macro procedures are simple debugging aids, intended
; mostly for debugging API code.
;------------------------------------------------------------

;-----------------------------------------------------------------
; This macro procedure displays a message, and optionally a DWORD
; value in decimal and hex, on the console, and does so without
; altering the flags or registers. The reparg macro allows this
; macro to accept quoted strings as well as string addresses.
;
; The justification for using the console instead of a message
; box, even for debugging a GUI app, is that a GUI app with
; problems that prevent it from terminating itself can usually
; be forced to terminate by simply closing the attached console.
; To add a console to a GUI app, build it as a console app.
;-----------------------------------------------------------------

SHOW MACRO message:REQ, ddval
    pushfd
    pushad
    IFNB <ddval>
      invoke printf, chr$("%s %d %xh%c"), reparg(message),
                     ddval, ddval, 10
    ELSE
      invoke printf, chr$("%s%c"), reparg(message), 10
    ENDIF
    popad
    popfd
ENDM

;-------------------------------------------------------------------
; This macro procedure displays, on the console, the system-defined
; message for the last-error code value returned by GetLastError,
; and does so without altering the flags or registers.
;
; Note that the last-error code is meaningful only immediately
; after an API function indicates, by its return value, that an
; error occurred.
;-------------------------------------------------------------------

SHOWLASTERROR MACRO
    pushfd
    pushad
    invoke GetLastError

    ;---------------------------------------------------------------
    ; Allocate a DWORD from the stack to serve as temporary storage
    ; for the buffer pointer returned by FormatMessage, and load its
    ; address into EBX.
    ;---------------------------------------------------------------

    sub esp, 4
    mov ebx, esp

    invoke FormatMessage, FORMAT_MESSAGE_FROM_SYSTEM or \
                          FORMAT_MESSAGE_ALLOCATE_BUFFER,
                          0,
                          eax,
                          0,
                          ebx,
                          1024,
                          0

    ;--------------------------------------------------------------
    ; The FORMAT_MESSAGE_ALLOCATE_BUFFER in the dwFlags parameter
    ; causes the function to allocate a buffer for the formatted
    ; message using the LocalAlloc function and place the address
    ; of the buffer at the address pointed to by the lpBuffer
    ; parameter. Since EBX is effectively a pointer to a pointer,
    ; to get the pointer to the message we must dereference EBX.
    ; Note that the DWORD PTR is necessary because the prototype
    ; for printf (in msvcrt.inc) specifies VARARG instead of DWORD,
    ; so MASM has no other way to know the size of the argument.
    ;--------------------------------------------------------------

    invoke printf, chr$("%s%c"), DWORD PTR [ebx], 10

    ;-----------------------------------------------------------------
    ; And to free the allocated buffer we must again dereference EBX.
    ;-----------------------------------------------------------------

    invoke LocalFree, [ebx]

    ;---------------------------------------------------------
    ; Restore the stack pointer, effectively deallocating the
    ; space on the stack, so the following pops will work.
    ;---------------------------------------------------------

    add esp, 4

    popad
    popfd
ENDM
